/*
 *  GQView
 *  (C) 2003 John Ellis
 *
 *  Authors:
 *    Support for Exif file format, written by Eric Swalens.    
 *    Modified by Quy Tonthat
 *

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __EXIF_H
#define __EXIF_H

#define EXIF_RESUNIT			1
#define EXIF_EXPOSURE_PROGRAM		2
#define EXIF_METERING_MODE		3
#define EXIF_LIGHT_SOURCE		4
#define EXIF_FLASH			5

#define EXIF_ORIENTATION_UNKNOWN	0
#define EXIF_ORIENTATION_TOP_LEFT	1
#define EXIF_ORIENTATION_TOP_RIGHT	2
#define EXIF_ORIENTATION_BOTTOM_LEFT	3
#define EXIF_ORIENTATION_BOTTOM_RIGHT	4
#define EXIF_ORIENTATION_LEFT_TOP	5
#define EXIF_ORIENTATION_RIGHT_TOP	6
#define EXIF_ORIENTATION_RIGHT_BOTTOM	7
#define EXIF_ORIENTATION_LEFT_BOTTOM	8

#define EXIF_RESUNIT_NOUNIT		1
#define EXIF_RESUNIT_INCH		2
#define EXIF_RESUNIT_CENTIMETER		3

#define EXIF_EXPOSURE_TIME_UNKNOWN	-1

#define EXIF_EXPOSURE_PROGRAM_UNKNOWN	0
#define EXIF_EXPOSURE_PROGRAM_MANUAL	1
#define EXIF_EXPOSURE_PROGRAM_NORMAL	2
#define EXIF_EXPOSURE_PROGRAM_APERTURE	3
#define EXIF_EXPOSURE_PROGRAM_SHUTTER	4
#define EXIF_EXPOSURE_PROGRAM_CREATIVE	5
#define EXIF_EXPOSURE_PROGRAM_ACTION	6
#define EXIF_EXPOSURE_PROGRAM_PORTRAIT	7
#define EXIF_EXPOSURE_PROGRAM_LANDSCAPE	8

#define EXIF_ISO_SENSIVITY_UNKNOWN	0

#define EXIF_APERTURE_UNKNOWN		0

#define EXIF_METERING_MODE_UNKNOWN		0
#define EXIF_METERING_MODE_AVERAGE		1
#define EXIF_METERING_MODE_CENTER_WEIGHTED	2
#define EXIF_METERING_MODE_SPOT			3
#define EXIF_METERING_MODE_MULTI_SPOT		4
#define EXIF_METERING_MODE_MULTI_SEGMENT	5
#define EXIF_METERING_MODE_PARTIAL		6
#define EXIF_METERING_MODE_OTHER		255


#define EXIF_LIGHT_SOURCE_UNKNOWN	0
#define EXIF_LIGHT_SOURCE_DAYLIGHT	1
#define EXIF_LIGHT_SOURCE_FLUORESCENT	2
#define EXIF_LIGHT_SOURCE_TUNGSTEN	3
#define EXIF_LIGHT_SOURCE_FLASH		10
#define EXIF_LIGHT_SOURCE_STANDARD_A	17
#define EXIF_LIGHT_SOURCE_STANDARD_B	18
#define EXIF_LIGHT_SOURCE_STANDARD_C	19
#define EXIF_LIGHT_SOURCE_D55		20
#define EXIF_LIGHT_SOURCE_D65		21
#define EXIF_LIGHT_SOURCE_D75		22
#define EXIF_LIGHT_SOURCE_OTHER		255

#define EXIF_FLASH_UNKNWON		255
#define EXIF_FLASH_NO			0
#define EXIF_FLASH_YES			1
#if 0
#define EXIF_FLASH_RETURN_NOT_DETECTED	5
#define EXIF_FLASH_RETURN_DETECTED	7
#endif

#define EXIF_FOCAL_LENGTH_UNKNOWN	0

#define EXIF_RATIONAL_UNKNOWN           ((ExifRational) {0,0})


typedef struct {
	unsigned int num;
	unsigned int den;
} ExifRational;

typedef struct {
	gchar *image_desc;
	gchar *make;
	gchar *model;
	unsigned int orientation;
	ExifRational resolution_x, resolution_y;
	unsigned int resolution_unit;
	gchar *software;
	gchar *date;
	gchar *copyright;
	ExifRational exposure_time;
	double fnumber;
	int exposure_program;
	int iso_sensivity;
	gchar *date_original;
	gchar *date_digitized;
	double shutter_speed;
	double aperture;
	double max_aperture;
	double exposure_bias;
	double subject_distance;
	int metering_mode;
	int light_source;
	int flash;
	double focal_length;
} ExifData;


ExifData *exif_read(const gchar *path);
void exif_free(ExifData *exif);

const gchar *exif_get_info_string(ExifData *exif, int info);

gchar *exif_rational_string(ExifRational *r);
gchar *exif_orientation_string(ExifData *exif);

#endif

