//    FlipFlop XMMS Plugin
//    Copyright 2003 Kevin Ogden
//    kogden@ualberta.ca
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// July 9. 2003
// Coding is a mess.  Added comments to make it somewhat understandable
// Since its probably never gonna be cleaned up.
  

#include <GL/gl.h>
#include <GL/glu.h>

#include <xmms/util.h>
#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>

#include <stdio.h>
#include <xmms/plugin.h>
#include <string.h>

#include <math.h>
#include <stdlib.h>

#include <unistd.h>

#ifndef PI
#define PI 3.14159
#endif

// dimensions of the board 12x12
#define BOARDSIZE 12
// The number of moving squares on the board
// should be less then BOARDSIZExBOARDSIZE
#define NUMSQUARES 128
#define HALFTHICK 0.04


// hold all the data needed for the sheet of 
// randomly moving squares
typedef struct{
  /* 2D array specifying which squares are where (to avoid collisions) */
  /* -1 means empty otherwise integer represents square index 0 - n-1 */
  int occupied[ BOARDSIZE ][ BOARDSIZE ];
  /* an array of xpositions of the squares */
  int xpos[ NUMSQUARES ];
  /* array of y positions of the squares */
  int ypos[ NUMSQUARES ];
  /* integer representing the direction of movement of a square */
  int direction[ NUMSQUARES ]; /* 0 not, 1 x+, 2 y+, 3 x-, 4 y-*/
  /* angle of moving square (during a flip) */
  float angle[ NUMSQUARES ]; 
  /* array of colors for a square.  rgb */
  /* eg. color[ 4 ][ 0 ] is the red component of square 4 */
  /* eg. color[ 5 ][ 2 ] is the blue component of square 5  */
  float color[ NUMSQUARES ][ 3 ];
  /* n is the number of square */
} randsheet;

// the main object in this visualization
randsheet MyRandSheet;

// the board spins...
double theta = 0.0;
/* amount which the square flips.  1 is a entire flip */
float flipspeed = 0.075;
/* relative height of camera */
float relheight = 1.0;
/* relative distace of camera from center */
float reldist = 0.7;
/* likelehood a square will attempt a move */
float energy = 5.0;

// is the music on or off??
int musicplaying = 0;


/* 0 is flipflop 1 is equilizer */
int playmode = 0;

// Forward declarations
static void flipflop_init(void);
static void flipflop_cleanup(void);
static void flipflop_about(void);
static void flipflop_configure(void);
static void flipflop_playback_start(void);
static void flipflop_playback_stop(void);
static void flipflop_render_freq(gint16 freq_data[2][256]);
void randsheet_initialize( randsheet *rs);
int randsheet_new_move( randsheet* rs );
void randsheet_move( randsheet *rs, float rot );
void randsheet_draw( randsheet *rs );
void setup_lights();
void display();
void initGL();
void determine_averages();
void change_colors();
void display_soundoff();
void color_soundoff();
void draw_sheet(void);

// Callback functions
VisPlugin flipflop_vtable = {
  0, // Handle, filled in by xmms
  0, // Filename, filled in by xmms

  0,                     // Session ID
  "Flipflop Plugin",       // description

  0, // # of PCM channels for render_pcm()
  1, // # of freq channels wanted for render_freq()

  flipflop_init,           // Called when plugin is enabled
  flipflop_cleanup,        // Called when plugin is disabled
  flipflop_about,          // Show the about box
  flipflop_configure,      // Show the configure box
  0,                     // Called to disable plugin, filled in by xmms
  flipflop_playback_start, // Called when playback starts
  flipflop_playback_stop,  // Called when playback stops
  0,                     // Render the PCM data, must return quickly
  flipflop_render_freq     // Render the freq data, must return quickly
};

// XMMS entry point
VisPlugin *get_vplugin_info(void)
{
  return &flipflop_vtable;
}

// Our worker thread
SDL_Thread *worker_thread;
enum { GO, STOP } thread_control;
SDL_mutex *mutex;
gint16 shared_freq_data[2][256];

// used for finite average array
// used to index the intensities buffer
// increments one each frame
int count = 0;
int flash = 0;
float ave_loud=0;

// used to determine what is loud and what is quite
// base relative to this average
float ave_intensity[ 256 ];
// used to calculate average
int sums[ 256 ];
// buffer holding past values of intensities
// also used to determine the averages
int intensities[ 256 ][ 1000 ];
int loudness;

float cur_buf[ 256 ];
float heights[ 128 ];
float fallspeed[ 128 ];

// visualization thread goes here to do the hard work
void worker_func(void)
{
  int i, j;
  // initialize stuff
  initGL();
  randsheet_initialize( &MyRandSheet );
  for( i=0; i<256; ave_intensity[ i++ ] = 0 );
  for( i=0; i<128; heights[ i++ ] = 0 );
  for( i=0; i<128; fallspeed[ i++ ] = 0 );
  for( i=0; i<256; cur_buf[ i++ ] = 0 );
  for( i=0; i<256; sums[ i++ ] = 0 );
  for( i = 0; i<256; i++ ){
    for( j = 0; j<1000; j++ ){
      intensities[ i ][ j ] = 200;
      sums[ i ]+= 200; 
    }
  }
  // main loop
while (thread_control == GO) {
  if( musicplaying )
    {

      /* sleep so its not a CPU hog */
      /* usleep( 30000 );  */
      /* or not :-) */

      if( rand() % 500 == 0 )
	{
	  playmode = !playmode;
	}
      
     ++count; 
      // change color based on frequency data
      change_colors();
      // this should give us a good idea what is
      // loud and what is quite
      determine_averages();
      // draw stuff
      display();

    }
  else
    {
      color_soundoff();
      display_soundoff();
    }
  }


}



static void flipflop_init(void) 
{

  if (SDL_Init (SDL_INIT_VIDEO) < 0) {
    printf ("Failed to initialize SDL\n");
    flipflop_vtable.disable_plugin (&flipflop_vtable);
    return;
  }
  // Start worker thread
  thread_control = GO;
  mutex = SDL_CreateMutex();
  memset(shared_freq_data, 0, sizeof(gint16) * 2 * 256);
  worker_thread = SDL_CreateThread ((void *) worker_func, NULL);


}

static void flipflop_cleanup(void)
{
  thread_control = STOP;
  SDL_WaitThread(worker_thread, NULL);
  SDL_DestroyMutex(mutex);
  SDL_Quit();
}

static void flipflop_about(void)
{
  printf("Flipflop plugin: About\n");
  printf("Flipflop version 0.1.0\n");
  printf("Kevin Ogden, July 2003\n");
  printf("kogden@ualberta.ca\n");
}

static void flipflop_configure(void)
{
  printf("Flipflop plugin: Configure not implemented.\n");
}

static void flipflop_playback_start(void)
{
  musicplaying = 1;
}

static void flipflop_playback_stop(void)
{
  musicplaying = 0;
}

static void flipflop_render_freq(gint16 freq_data[2][256])
{
 int channel, bucket;
 int on = 0;
 SDL_mutexP(mutex);
 for (channel = 0; channel < 2; channel++) 
   {
     for (bucket = 0; bucket < 256; bucket++) 
       {
	 shared_freq_data[channel][bucket] = freq_data[channel][bucket];
	 on = on | shared_freq_data[ 0 ][ bucket ];
       }
   }
 SDL_mutexV(mutex);

  if( on )
    musicplaying = 1;
  else
    musicplaying = 0;
}

// determines the average intensity for each frequency band over the last
// 1000 frames.
void determine_averages( )
{
  int i;
  SDL_mutexP(mutex);
  // sums
  for( i=0; i<256; i++ )
    {
      /* remove old sum */
      sums[ i ] -= intensities[ i ][ count%1000 ];
      /* determine current intensity */
      intensities[ i ][ count %1000 ] = shared_freq_data[0][i];
      /* add the new value to the sum */
      sums[ i ] += intensities[ i ][ count %1000 ];
      /* determin the average intensity */
      ave_intensity[ i ] = ( float )sums[ i ] / 1000;
      /* current w.r.t. average */
      if( ave_intensity[ i ] > 0 )
	cur_buf[ i ] = (float)intensities[ i ][ count %1000 ]/ave_intensity[ i ];
      else
	cur_buf[ i ] = 999;
    }
  /* overall loudness (all freq. bands combined */
  ave_loud = 0;
  loudness = 0;
  for( i =0; i<256; i++ )
    {
      loudness += intensities[i++][count%1000];
      ave_loud += ave_intensity[ i++ ];
    }
  /* is it loud enough to flash ? */
  if( loudness > 20000 )
    {
      if( (float) loudness/ave_loud > 3.0 )
	{
	  /* determines length of flash */
	  flash = ( loudness/ave_loud - 3.0 )*5; 
	}
    }
  SDL_mutexV(mutex);
}

// change colors based on how loud the current frequency data is compared to
// the averages.
// Colors are split into three:
// RED - BASS
// GREEN - MID
// BLUE - HIGH
//
// Basically...
// If LOUD ENOUGH 
//    MAKE BRIGHT
// Else 
//    Dim-Down
void change_colors( )
{
  int i;
  if( flash > 0 )
    {
      for( i = 0; i < 42; i++ )
	{
      MyRandSheet.color[ i ][ 0 ] = 1 - flash*0.1;  
      MyRandSheet.color[ i ][ 1 ] = 0; 
      MyRandSheet.color[ i ][ 2 ] = 0;	  
	} 
      for( i = 42; i < 88; i++ ) 
	{
      MyRandSheet.color[ i ][ 0 ] = 0;  
      MyRandSheet.color[ i ][ 1 ] = 1 - flash*0.1; 
      MyRandSheet.color[ i ][ 2 ] = 0;	  
	}
      for( i = 88; i < 128; i++ ) 
	{
      MyRandSheet.color[ i ][ 0 ] = 0;  
      MyRandSheet.color[ i ][ 1 ] = 0; 
      MyRandSheet.color[ i ][ 2 ] = 1 - flash*0.1;
	}
    }
  else
    {

 for( i =0; i<42; i++ )
    {
	  // make bright
      if( cur_buf[i+128] > 1.5 )
	{
	  MyRandSheet.color[ i ][ 0 ] = 1;  
	  MyRandSheet.color[ i ][ 1 ] = 0; 
	  MyRandSheet.color[ i ][ 2 ] = 0;
	}
      else
	{
	  // / dim down
	  if(  MyRandSheet.color[ i ][ 0 ] > 0.25 )
	    {
	      MyRandSheet.color[ i ][ 0 ] = MyRandSheet.color[ i ][ 0 ]*0.95;
	    }
	}
    }
  for( i =42; i<88; i++ )
    {
      // make bright
      if( cur_buf[i+128] > 1.5 )
	{
	  MyRandSheet.color[ i ][ 0 ] = 0;  
	  MyRandSheet.color[ i ][ 1 ] = 1; 
	  MyRandSheet.color[ i ][ 2 ] = 0;
	}
      else
	// dim down
	{
	  if(  MyRandSheet.color[ i ][ 1 ] > 0.25 )
	    MyRandSheet.color[ i ][ 1 ] = MyRandSheet.color[ i ][ 1 ]*0.95;
	}
    }
  for( i =88; i<128; i++ )
    {
      // make bright
      if( cur_buf[i+128] > 1.5 )
	{
	  MyRandSheet.color[ i ][ 0 ] = 0;  
	  MyRandSheet.color[ i ][ 1 ] = 0; 
	  MyRandSheet.color[ i ][ 2 ] = 1;
	}
      // dim down
      else
	{
	  if( MyRandSheet.color[ i ][ 2] > 0.25 )
	    MyRandSheet.color[ i ][ 2 ] = MyRandSheet.color[ i ][ 2 ]* 0.95;
	}
    }
    }
}

// draws one frame of MyRandSheet       
void display() {
  int i;

  if( flash > 0 )
    {
      glClearColor( flash*0.1, flash*0.1, flash*0.1, 0.0f);
      flash--;
    }
  else
    {
      glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    }
  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.5);
  glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.1);
  float amb[ 4 ] = { 0.3, 0.3, 0.3, 1 };
  glLightfv(GL_LIGHT0, GL_AMBIENT, amb);
  

  /** setup perspectif */
  glTranslatef(0.0, 0.0, -BOARDSIZE*1.2 );
  glRotatef(30.0, 1.0, 0.0, 0.0);
  glRotatef(theta*100, 0.0, 1.0, 0.0);
  glTranslatef(-0.5*BOARDSIZE, 0.0, -0.5*BOARDSIZE);
  //  printf( "loudness %i\n", loudness );
  //printf( "tries: %i\n" , NUMSQUARES*energy );

  // *** WARNING - ARBITRARY NUMBERS PUT HERE THAT SEEM TO WORK ***
  // (for most songs)
  // if loud enough let some squares flip over..
  if( loudness > 15000 )
    {
      for( i=0; i < (NUMSQUARES*loudness/150000) ; i++ )
	randsheet_new_move( &MyRandSheet );
    }
  // base the flipspeed on the loudness
  // divide by a big number so they dont flip too fast
  if( ave_loud > loudness*2 )
    { 
      flipspeed = 0.005;
    }
  else
    {
      flipspeed = 0.005 + ((float)loudness/3000000);
    }
  randsheet_move( &MyRandSheet, flipspeed * 3.14159 );
  randsheet_draw( &MyRandSheet );
 if( ave_loud > loudness*2 )
   {
     flash = 0;
   }
 else{
  if( flash == 0 )
    {
      theta += .002;
    }
  else
    {
      theta -= .002*flash*0.25;
      flipspeed *= 2;
    }
 }
  glFinish(); 
  SDL_GL_SwapBuffers();
   
}

void color_soundoff()
{
  int i;
  for( i = 0; i<128; i++ )
    {
      MyRandSheet.color[ i ][ 0 ] = 1 - (float)i/128;  
      MyRandSheet.color[ i ][ 1 ] = 0; 
      MyRandSheet.color[ i ][ 2 ] = (float)i/128;
    }      

}


void display_soundoff()
{
  int i;

  for( i = 0; i<256; i++ )
    {
      cur_buf[ i ] = 0;
    }
  glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
  glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

  
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, 0.5);
  glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, 0.1);
  float amb[ 4 ] = { 0.3, 0.3, 0.3, 1 };
  glLightfv(GL_LIGHT0, GL_AMBIENT, amb);
  

  /** setup perspectif */
  glTranslatef(0.0, 0.0, -BOARDSIZE );
  glRotatef(30, 1.0, 0.0, 0.0);
  glRotatef(theta*100, 0.0, 1.0, 0.0);
  glTranslatef(-0.5*BOARDSIZE, 0.0, -0.5*BOARDSIZE);
  //  printf( "loudness %i\n", loudness );
  //printf( "tries: %i\n" , NUMSQUARES*energy );

  for( i=0; i < NUMSQUARES ; i++ )
    randsheet_new_move( &MyRandSheet );

  // base the flipspeed on the loudness
  // divide by a big number so they dont flip too fast
  flipspeed = 0.02;
  randsheet_move( &MyRandSheet, flipspeed * 3.14159 );
  randsheet_draw( &MyRandSheet );
  theta += .001;
  glFinish(); 
  SDL_GL_SwapBuffers();
}

/* configure lighting */
void setup_lights(void) 
{
  GLfloat position0[] = { 10.0, 2.0, 10.0, 1.0 }; 
  glEnable(GL_LIGHTING);
  glLightfv(GL_LIGHT0, GL_POSITION, position0);
  glEnable(GL_LIGHT0);
}

void initGL( )
{
  // Create window for OpenGL output
  if (!SDL_SetVideoMode( 640, 480, 32, SDL_OPENGL)) {
    printf ("Failed to create window\n");
    flipflop_vtable.disable_plugin (&flipflop_vtable);
    return;
  }
  // chose those numbers to make a black outline around the window
  glViewport(0, 0, 640, 480);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(45, 640/480, 1, 30);
  glMatrixMode(GL_MODELVIEW);
  glClear(GL_COLOR_BUFFER_BIT  );
  glClearColor(0.0, 0.0, 0.0, 0.0);
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
  glEnable(GL_COLOR_MATERIAL);
  setup_lights();
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);

}

/*** ADDED RANDSHEET FUNCTIONS ***/
// These functions were originally member functions in a C++ 
// randsheet class.  Converted to C for portablity
// i.e. this, and a xscreensaver


void draw_sheet( void )
{

  glBegin( GL_QUADS );
  glNormal3f( 0, -1, 0 );
  glVertex3f( HALFTHICK,  -HALFTHICK,  HALFTHICK );
  glVertex3f( 1-HALFTHICK,   -HALFTHICK, HALFTHICK );
  glVertex3f( 1-HALFTHICK, -HALFTHICK,  1-HALFTHICK);
  glVertex3f( HALFTHICK, -HALFTHICK, 1-HALFTHICK );
  
  /* back */
  glNormal3f( 0, 1, 0 );
  glVertex3f( HALFTHICK, HALFTHICK, 1-HALFTHICK );
  glVertex3f( 1-HALFTHICK, HALFTHICK,  1-HALFTHICK);
  glVertex3f( 1-HALFTHICK,   HALFTHICK, HALFTHICK );
  glVertex3f( HALFTHICK,  HALFTHICK,  HALFTHICK );
  
  /* 4 edges!!! weee.... */
  glNormal3f( 0, 0, -1 );
  glVertex3f( HALFTHICK, HALFTHICK, HALFTHICK );
  glVertex3f( 1-HALFTHICK, HALFTHICK, HALFTHICK );
  glVertex3f( 1-HALFTHICK, -HALFTHICK, HALFTHICK );
  glVertex3f( HALFTHICK, -HALFTHICK, HALFTHICK );
  glNormal3f( 0, 0, 1 );
  glVertex3f( HALFTHICK, HALFTHICK, 1-HALFTHICK );
  glVertex3f( HALFTHICK, -HALFTHICK, 1-HALFTHICK );
  glVertex3f( 1-HALFTHICK, -HALFTHICK, 1-HALFTHICK );
  glVertex3f( 1-HALFTHICK, HALFTHICK, 1-HALFTHICK );
  glNormal3f( 1, 0, 0 );
  glVertex3f( 1-HALFTHICK, HALFTHICK, 1-HALFTHICK );
  glVertex3f( 1-HALFTHICK, -HALFTHICK, 1-HALFTHICK );
  glVertex3f( 1-HALFTHICK, -HALFTHICK, HALFTHICK );
  glVertex3f( 1-HALFTHICK, HALFTHICK, HALFTHICK );
  glNormal3f( -1, 0, 0 );
  glVertex3f( HALFTHICK, HALFTHICK, 1-HALFTHICK );
  glVertex3f( HALFTHICK, HALFTHICK, HALFTHICK );
  glVertex3f( HALFTHICK, -HALFTHICK, HALFTHICK );
  glVertex3f( HALFTHICK, -HALFTHICK, 1-HALFTHICK );
  glEnd();
}

void randsheet_initialize( randsheet *rs ){
  int i, j, index;
      index = 0;
     /* put the moving sheets on the board */
      for( i = 0; i < BOARDSIZE; i++ )
	{
	  for( j = 0; j < BOARDSIZE; j++ )
	    {
	      /* initially fill up a corner with the moving squares */
	      if( index < NUMSQUARES )
		{
		  rs->occupied[ i ][ j ] = index;
		  rs->xpos[ index ] = i;
		  rs->ypos[ index ] = j;
		  /* have the square colors start out as a pattern */
		  rs->color[ index ][ 0 ] = ( index < 43 ? 0.2 : 0 );
		  rs->color[ index ][ 1 ] = ( index > 42 && index < 89 ? 0.2 : 0 );
		  rs->color[ index ][ 2 ] = ( index > 88 ? 0.2 : 0 );
		 ++index;  
		}
		/* leave everything else empty*/
	      else
		{
		  rs->occupied[ i ][ j ] = -1;
		}
	    }
	}
	/* initially everything is at rest */
      for( i=0; i<NUMSQUARES; i++ )
	{
	  rs->direction[ i ] = 0;
	  rs->angle[ i ] = 0;
	}
    }
       
/* Pick and random square and direction and try to move it. */
/* May not actually move anything, just attempt a random move. */
/* Returns true if move was sucessful. */
/* This could probably be implemented faster in a dequeue */
/* to avoid trying to move a square which is already moving */
/* but speed is most likely bottlenecked by rendering anyway... */
  int randsheet_new_move( randsheet* rs )
    {
      int i, j;
      int num;
      /* pick a random square */
      num = rand( ) % NUMSQUARES;
      i = rs->xpos[ num ];
      j = rs->ypos[ num ];
      /* pick a random direction */
      int dir = ( rand( )% 4 ) + 1;
      
      if( rs->direction[ num ] == 0 )
	{
	  switch( dir )
	    {
	    case 1:
	      /* move up in x */
	      if( ( i + 1 ) < BOARDSIZE )
		{
		  if( rs->occupied[ i + 1 ][ j ] == -1 )
		    {
		      rs->direction[ num ] = dir;
		      rs->occupied[ i + 1 ][ j ] = num;
		      rs->occupied[ i ][ j ] = -1;
		      return 1;
		    }
		}
	      return 0;
	      break;
	    case 2:
	      /* move up in y */
	      if( ( j + 1 ) < BOARDSIZE )
		{
		  if( rs->occupied[ i ][ j + 1 ] == -1 )
		    {
		      rs->direction[ num ] = dir;
		      rs->occupied[ i ][ j + 1 ] = num;
		      rs->occupied[ i ][ j ] = -1;
		      return 1;
		    }
		}
	      return 0;
	      break;
	    case 3:
	      /* move down in x */
	      if( ( i - 1 ) >= 0 )
		{
		  if( rs->occupied[ i - 1][ j ] == -1 )
		    {
		      rs->direction[ num ] = dir;
		      rs->occupied[ i - 1][ j ] = num;
		      rs->occupied[ i ][ j ] = -1;
		      return 1;
		    }
		}
	      return 0;
	      break;
	    case 4:
	      /* move down in y */
	      if( ( j - 1 ) >= 0 )
		{
		  if( rs->occupied[ i ][ j - 1 ] == -1 )
		    {
		      rs->direction[ num ] = dir;
		      rs->occupied[ i ][ j - 1 ] = num;
		      rs->occupied[ i ][ j ] = -1;
		      return 1;
		    }
		}
	      return 0;
	      break;
	    default:
	      break;
	    } 
	}
      return 0;
    }

/*   move a single frame.  */
/*   Pass in the angle in rads the square rotates in a frame. */
  void randsheet_move( randsheet *rs, float rot )
    {
      int i, j, index;
      for( index = 0 ; index < NUMSQUARES; index++ )
	{
	  i = rs->xpos[ index ];
	  j = rs->ypos[ index ];
	  switch( rs->direction[ index ] )
	    {
	    case 0:
	      /* not moving */
	      break;
	    case 1:
	    /* move up in x */
	      rs->angle[ index ] += rot;
	      /* check to see if we have finished moving */
	      if( rs->angle[ index ] >= PI  )
		{
		  rs->xpos[ index ] += 1;
		  rs->direction[ index ] = 0;
		  rs->angle[ index ] = 0;
		}	 
	      break;
	    case 2:
	    /* move up in y */
	      rs->angle[ index ] += rot;  
	      /* check to see if we have finished moving */
	      if( rs->angle[ index ] >= PI  )
		{
		  rs->ypos[ index ] += 1;
		  rs->direction[ index ] = 0;
		  rs->angle[ index ] = 0;
		}	 
	      break;
	    case 3:
	    /* down in x */
	      rs->angle[ index ] += rot;  
	      /* check to see if we have finished moving */
	      if( rs->angle[ index ] >= PI  )
		{
		  rs->xpos[ index ] -= 1;
		  rs->direction[ index ] = 0;
		  rs->angle[ index ] = 0;
		}	 
	      break;
	    case 4:
	    /* up in x */
	      rs->angle[ index ] += rot;
 	      /* check to see if we have finished moving */
	      if( rs->angle[ index ] >= PI  )
		{
		  rs->ypos[ index ] -= 1;
		  rs->direction[ index ] = 0;
		  rs->angle[ index ] = 0;
		}	 
	      break;
	    default:
	      break;
	    }
	}
    }

    /* draw all the moving squares  */
  void randsheet_draw( randsheet *rs )
    {
      int i, j;
      int index;
      /* for all moving squares ... */
      for( index = 0; index < NUMSQUARES; index++ )
	{
	  /* set color */
	  glColor3f( rs->color[ index ][ 0 ], 
		     rs->color[ index ][ 1 ], 
		     rs->color[ index ][ 2 ] );
		     /* find x and y position */

	  i = rs->xpos[ index ];
	  j = rs->ypos[ index ];
	  glPushMatrix();
/* 	  glTranslatef( 0, ((float) (i+j)/BOARDSIZE)*sin( theta*100 ), 0 ); */

	  /* force from equilizer (whatever that means) */
	  fallspeed[ index] += (cur_buf[ index ]/((heights[ index ]+0.25)*(heights[ index]+0.25)))*0.005;
	  fallspeed[ index ] -= 0.03;
	  heights[index]+=fallspeed[ index ];
	  /* gravity */
	  if( heights[ index ] <= 0 )
	    {
	      heights[ index ]=0;
	      fallspeed[ index ] = 0;
	    }
	  switch( rs->direction[ index ] )
	    {
	    case 0:
	      
	    /* not moving */
	      /* front */
	      glTranslatef( i, heights[ index ], j );
	      break;
	    case 1:
	      glTranslatef( i+1, heights[ index ], j );
	      glRotatef( 180 - rs->angle[ index ]*180/PI, 0, 0, 1 );
	      
	      break;
	    case 2:
	      glTranslatef( i, heights[ index] , j+1 );
	      glRotatef( 180 - rs->angle[ index ]*180/PI, -1, 0, 0 );
	      
	      break;
	    case 3:
	      glTranslatef( i, heights[ index ], j );
	      glRotatef( rs->angle[ index ]*180/PI, 0, 0, 1 );
	      break;
	    case 4:

	      glTranslatef( i, heights[ index ], j );
	      glRotatef( rs->angle[ index ]*180/PI, -1, 0, 0 );
	      break;
	    default:
	      break;
	    }
	  draw_sheet();
	  glPopMatrix();

	}
    }	  
/**** END RANDSHEET FUNCTIONS ***/

    // Get the bass amplitude
    // SDL_mutexP(mutex);
    // height = shared_freq_data[0][0] / 1024.0;
    // SDL_mutexV(mutex);
